/*
 * Gstart.c
 *
 * Created: 27.02.2017 14:42:12
 *  Author: dsosnowski
 */ 


#define F_CPU 1000000
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "wdog.h"

//------------------------------------------------------------------
#define OUT_PIN				0		//H
#define PWM_TIM				TIMER0A
#define LED_PIN				1		//H
#define REG_PIN				4		//A
#define REG_CH				2		//ADC

#define PWM_RESOLUTION		255
#define PWM_START_VALUE		10
#define PWM_STOP_VALUE		(PWM_RESOLUTION - 10)

#define LED_DUTY			10
#define LED_PERIOD			150

void HardInit(void);
void AnalogInit(void);
void PwmInit(void);
void PwmEnable(void);
void PwmDisable(void);


//------------------------------------------------------------------
struct {
	uint8_t duty;
	uint8_t delay;
	uint8_t reg;
} pwm;

struct {
	uint16_t duty;
	uint8_t enable;
} led;


//------------------------------------------------------------------
int main(void)
{	
	HardInit();
	AnalogInit();
	PwmInit();
	
	pwm.duty = PWM_START_VALUE;
	pwm.delay = 0xFF;
	
    while(1)
    {
		_delay_ms(1);
		
		pwm.reg = (ADCH >> 2);
		if (pwm.delay < pwm.reg){
			pwm.delay++;
		} else {
			pwm.delay = 0;
			
			if (pwm.duty < PWM_STOP_VALUE) {
				pwm.duty++;				
				OCR0A = (0xFF - pwm.duty);
				PORTB &= ~(1<<OUT_PIN);					
				PwmEnable();
				led.enable = 1;
			} else {
				PORTB |= (1<<OUT_PIN);
				PwmDisable();
				led.enable = 0;				
			}
		}
		
		if (led.enable > 0){
			if (led.duty < LED_DUTY){ 
				PORTB |= (1<<LED_PIN);
			} else {
				PORTB &= ~(1<<LED_PIN);			
			}
			led.duty++;
			
			if (led.duty >= LED_PERIOD){
				led.duty = 0;
			}
		} else {
				PORTB |= (1<<LED_PIN);			
		}
		
		WdtReset();
    }
}

//------------------------------------------------------------------
void HardInit(void){
	PORTB = 0;
	DDRB = (1<<LED_PIN | 1<<OUT_PIN);
	ACSR = (1<<ACD);
	PRR = (1<<PRTIM1 | 1<<PRUSI);
	WdtSet(7);
}

//----------------------------------
void AnalogInit(void){
	DIDR0 = (1<<ADC2D);
	ADCSRB = 0;
	ADCSRA = (1<<ADEN | 1<<ADATE | 1<<ADPS2 | 1<<ADPS1);
	ADMUX = (1<<ADLAR | REG_CH);
	ADCSRA |= (1<<ADSC);
}

//----------------------------------
void PwmInit(void){	
	TCCR0A = (1<<WGM01 | 1<<WGM00);		//fast pwm
	TCCR0B = (1<<CS01);
}

//----------------------------------
void PwmEnable(void){
	TCCR0A = (1<<WGM01 | 1<<WGM00 | 1<<COM0A1 | 1<<COM0A0);
}

//----------------------------------
void PwmDisable(void){
	TCCR0A = (1<<WGM01 | 1<<WGM00);
}
